#!/usr/bin/perl

#
# Copyright (C) 2013 Frederic Vecoven
#
# This file is part of trs_hard
#
# trs_hard is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# trs_hard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#


use strict;
use Digest::CRC;

my $project_dir = $ARGV[0];

my $file = $project_dir . "FatFS.hex";
my $file_out = $project_dir . "crc.inc";

if (! -f $file) {
	# FatFS.hex doesn't exist yet
	make_inc($file_out, 0);
	exit 0;
}

my $linenum = 0;
my @data;
my $addr = 0;
my $i;

for ($i = 0; $i < 0x10000; $i++) {
    $data[$i] = 0xff;
}

open(my $F, "< $file") or die "Can't open $file : $!\n";
while (<$F>) {
    $linenum++;
    s/\r$//;
    chomp;
    if (/^:/) {
	my $line = $_;
	my $binrec = pack('H*', substr($_, 1));
	my $check = unpack("%8C*", $binrec);
	die "Invalid checksum at line $linenum" unless $check == 0;

	my ($addr2, $type, $data2) = unpack("x n C X4 C x3 /a", $binrec);

	if ($type == 4) {
	    # upper address
	    $addr &= 0x0000ffff;
	    $addr |= ($addr2 << 16);
	} elsif ($type == 0) {
	    $addr &= 0xffff0000;
	    $addr |= $addr2;
	    if ($addr < 0x10000) {
		$i = 0;
		map( $data[$addr + $i++] = hex($_),
		     unpack('H2' x length($data2), $data2));
	    }
	}
    }
}
close($F);

my $digest = Digest::CRC->new(type => "crc16");
for ($i = 0x800; $i < 0xFFFE; $i++) {
    $digest->add(chr $data[$i]);
}

make_inc($file_out, $digest->digest);

exit 0;


sub make_inc {
	my $file = shift;
	my $crc = shift;
	
	open(my $OUT, " > $file") or die "Can't create $file : $!\n";
	printf $OUT "THECRC\tequ\t0x%04x\n", $crc;
	close($OUT);
	
	print "Generated $file_out successfully\n";
}
